---
title: Introduction to OO Programming Open Educational Resource
description: These resources are provided free of charge for instructors and students to use in their study of object oriented programming
published: true
date: 2022-08-31T12:46:41.035Z
tags: 
editor: markdown
dateCreated: 2022-05-02T19:44:40.050Z
---

This OER resource of information and activities will help you learn how to program in an Object-Oriented fashion using the Java programming language.


# Tabs here{.tabset}


## Plan of Study
### Tabs here{.tabset}
#### Weeks 1-2
##### Tools used in this course
  - [Tutorials](/tools/jupyterLabIntro)
  - [Programming with a Text Editor](/tools/editors)
  - [Programming with VS Code](/tools/vsCodeIntro)
  - [List of other tools](/tools)

##### Vocabulary
  - [Classes, Objects and Instances](/ooConcepts/classes)
  - [Instance Methods](/ooConcepts/methods)
  - [Instance Variables](/ooConcepts/variables)
  - [Class Methods and Variables](/ooConcepts/classMembers)
  - [Encapsulation](/ooDesign/encapsulation)

##### Beginning Java
  - [Java Hello World](/java/helloWorld)
  - [Java Syntax](/java/syntaxGuide)
  - [Java Compilation](/java/interpretedLanguage)
  - [The main method](/java/mainMethod)
  - [Stdin with `Scanner`](/inputOutput/stdin)
  - [Stdout with `System.out`](/inputOutput/stdout)

  
#####  OO Programs
  - [Procedural Programming vs OO Programming](/ooConcepts/proceduralOOCompare) 
  - [References](/ooConcepts/references)
  - [Message Passing](/ooConcepts/messagePassing)
  - [Objects composed of other Objects](/ooDesign/aggregation)
  - [Accessors and Mutators](/ooDesign/accessorsMutators)
  - [Access Control](/java/accessControls)
  - [Creating objects using constructors](/ooDesign/constructors)

#### Weeks 3-7
#####  I/O

- [Headless Computing](/inputOutput/headlessComputing)
- [Streams](/inputOutput/streams)
- [Stdin (review)](/inputOutput/stdin)
- [Stdout (review)](/inputOutput/stdout)
- [Text Files](/inputOutput/textFiles)
- [Binary Files](/inputOutput/binaryFiles)
- [Object Serialization](/inputOutput/serialization)

#####  Errors

- [Software Errors](/errors/errorsVsBugs)
- [Debugging](/errors/debugging)
- [Exceptions](/errors/exceptions)

#####  Testing

- [Software Testing Intro](/testing/testing)
- [Unit Testing](/testing/unitTesting)
- [Identifying Test Cases](/testing/testCases)
- [Unit testing with JUnit](/tools/junit)


##### Collections

- [Collections and Data Structures](/dataStructures/collectionsDataStructures)
- [Java Collections](/dataStructures/collections)
- [Collections Framework](/dataStructures/collectionsFramework)

#### Weeks 8-12
#####  Best Practices
   - [Information Hiding/ Encapsulation](/ooDesign/encapsulation)
  - [Single Responsibility Principle](/ooDesign/singleResponsibility)
  - [Avoid duplicate code](/ooDesign/duplicateCode)
  - [Organization and Naming Conventions](/tools/checkstyle)

#####  Polymorphism

- [Polymorphism](/ooDesign/polymorphism)
- [Polymorphism via Interfaces](/ooDesign/interfaces)
- [Polymorphism via Inheritance](/ooDesign/inheritance)

#####  Event Driven Programming

  - [Events](/eventDrivenProgramming/events)
  - [Event Handlers](/eventDrivenProgramming/eventHandlers)
  - [UX/UI Design Intro ](/eventDrivenProgramming/uxDesignBasics)
  - [Graphical User Interface ](/eventDrivenProgramming/gui)
 

#### Advanced

##### Generics

The use of parameterized types (Generics)  increases the type-safety of an OO program.  In this section we learn how to create our own Generic types and use them. 

 - [Intro to Generics](generics/introToGenerics)
 


#####  Functional Programming

Functional programming has many similarities with OO programming.  In this section you will learn about the functional programming conventions that are availble in the Java programming language and how they can be used to improve the overall design of your OO program.


- [What is functional programming](/functionalProgramming/functionalProgramming)
- [Lambda expressions as GUI listeners](/functionalProgramming/lambdaListeners)
- [Filters](/functionalProgramming/functionalFilters)
- [Maps](/functionalProgramming/functionalMaps)
- [Reductions](/functionalProgramming/functionalReductions)
- [Functional Collection Processing](/functionalProgramming/functionalCollectionProcessing)
## Topics

- [Programming Tools](/tools)
- [Java Specifics](/java)
- [Object-Oriented Concepts](/ooConcepts)
- [User Interaction: Input and Output](/inputOutput)
- [Object-Oriented Data Structures](/dataStructures)
- [Error Handling](/errors)
- [Object-Oriented Design](/ooDesign)
- [Event Driven Programming](/eventDrivenProgramming)
- [Object-Oriented Testing](/testing)
- [Introduction to Functional Programming](/functionalProgramming)

## Practice Activities

- [Java Syntax](/practiceActivities/java)
- [Object-Oriented Concepts](/practiceActivities/ooConcepts)
- [User Interaction: Input and Output](/practiceActivities/inputOutput)
- [Object-Oriented Data Structures](/practiceActivities/dataStructures)
- [Error Handling](/practiceActivities/errors)
- [Object-Oriented Design](/practiceActivities/ooDesign)
- [Event Driven Programming](/practiceActivities/eventDrivenProgramming)
- [Object-Oriented Testing](/practiceActivities/testing)
- [Introduction to Functional Programming](/practiceActivities/functionalProgramming)

## About the Resource
#### Authors
This resource was created by Dr. J. McCuaig and M. Asch.   C. Norrie and H. Al-Rubaye were employed  as summer students to author wiki pages and create activity solutions.  

Students from the Fall 2022 offering of Intro to Object-Oriented Programming at the University of Guelph provided valuable bug reports and feedback about the resource.  Thanks also to OO students who contributed code including M. Schmid, B. Sengupta, M. Ladyjenko, and R. Blotsky.

This project wasmade possible with funding from the University of Guelph OER support program and the Government of Ontario and through eCampusOntario’s support of the Virtual Learning Strategy. To learn more about the Virtual Learning Strategy visit: https://vls.ecampusontario.ca.


#### Learning Outcomes
Once you master the concepts contained in these resources you will be able to:
 - Differentiate between procedural and object-oriented paradigms
 - Design and implement classes for an object-oriented program demonstrating correct use of encapsulation, constructors, method overloading, class invariants, accessors, mutators, instance variables and class variables.
 - Construct class hierarchies that maximize code reuse through inheritance while accommodating differences through method overriding.
 -  Describe polymorphism and identify situations in which it is used in an OO program. .
 -  Use polymorphism, abstract methods/classes, and interfaces effectively to produce reusable, clean code.
 -  Read and understand class diagrams written in UML (Unified Modeling Language)
 -  Compare event-driven programming with  process-driven programming.
- [Information for Instructors](/aboutOER/instructorInfo)
- [Resources and References](/aboutOER/references)


## Licensing

###### This SCI-OER Java resource is intended to help novice programmers learn to program in an object-oriented fashion using the Java programming language.

**Copyright (C) 2023  Dr. J. McCuaig and M. Asch**

This resource is free: you can redistribute it and/or modify it under the terms of the GNU  General Public License as published  by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

This resource is distributed in the hope that it will be useful,but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the [GNU  General Public License](/aboutOER/license) for more details.